.SH
References
.LP
.IP "[1]"
Gosling, J. and Rosenthal, D. 
``A Window-Manager for Bitmapped Displays and UNIX,''
to be published in \fIMethodology of Window-Managers,\fP
F. R. A. Hopgood et al (eds) North-Holland.
.IP "[2]"
Newman, R., Rosenthal, D., Gettys, J.
``User Extensible Streams,''
In preparation.
.IP "[3]"
Richie, D. M., ``A Stream Input-Output System,''
\fIAT&T Bell Laboratories Technical Journal\fP,
Vol. 63, No. 8, Part 2, pp. 1897, October 1984.
.IP "[4]"
Birrell, A. D. and Nelson, B. J.,
``Implementing Remote Procedure Calls,''
\fITransactions on Computer Systems\fP,
vol. 2, no. 1, February 1984.
.IP "[5]"
Nelson, B. J., ``Remote Procedure Call,''
\fITechnical Report CSL-81-9\fP,
Xerox Palo Alto Research Center, 1981.
.IP "[6]"
``Sun Remote Procedure Call Specification,''
Sun Microsystems, Inc. Technical Report 1984.
.IP "[7]"
Souza, R. J. and Miller, S. P.,
``UNIX and Remote Procedure Calls: A Peaceful Coexistence?,''
Project Athena Internal Paper, 1985.
