/*
 *	$Source: /u1/X/libis/RCS/vssite.h,v $
 *	$Header: vssite.h,v 1.1 86/11/17 14:35:12 swick Rel $
 */

#include "is-copyright.h"

/* vssite.h
 *
 * This is where any machine-specific information should be stored so that
 * different sites can have control over where things live.
 *
 * Copyright (c) 1986, Integrated Solutions, Inc.
 *
 */

#define DEFAULT_FONT_DIRECTORY	"/usr/lib/X/font/"
#define DEFAULT_FONT_SUFFIX	".onx"
