/*
 *	$Source: /u1/Xr/usr/contrib/RB/RCS/main.h,v $
 *	$Header: main.h,v 1.1 86/12/17 08:54:50 swick Exp $
 */

/*************************************<+>*************************************
 *****************************************************************************
 **
 **   File:        main.h
 **
 **   Project:     X-ray Toolbox
 **
 **   Description: This file contains the defines, strucutures, and data
 **                necessary for main.c of the icon builder.
 **
 **   *******************************************************************
 **   * (c)  Copyright Hewlett-Packard Company, 1986.  All rights are   *
 **   * reserved.  Copying or other reproduction of this program except *
 **   * for archival purposes is prohibited without the prior written   *
 **   * consent of Hewlett-Packard Company.                             *
 **   *******************************************************************
 **
 **
 **   ------------------------ MODIFICATION RECORD   ------------------------
 *
 * $Log:	main.h,v $
 * Revision 1.1  86/12/17  08:54:50  swick
 * Initial revision
 * 
 *
 *****************************************************************************
 *************************************<+>*************************************/

static Window mainWindowId;
static xrWindowData mainWindowData;


static xrTitleBarInfo titleBarInfo =
{
   0,				/* windowId */
   {0, 0, 0, 0},		/* editor rectangle */
   0,				/* editor state     */
   -1, -1,			/* editor colors    */
   NULL,			/* editor font      */
   "Raster Builder",		/* title name	    */
   NULL,			/* gadget 1	    */
   NULL				/* gadget 2	    */
};


xrRasterEditInfo rasterEditInfo =
{
   0,				/* windowId         */
   {0, 0, 0, 0},		/* editor rectangle */
   XrVISIBLE | XrSENSITIVE,	/* editor state     */
   -1, -1,			/* editor colors    */
   0,                           /* pixel color      */
   8,                           /* pixel size       */
   { 24, 24 },                  /* view region      */
   { 64, 64, 0, 0 }		/* pixmap info      */
};


static xrEditor * mainRasterEdit;
static xrEditor * mainTitleBar;

static xrMenu * mainMenu;
static INT8 * menuItems [8] =
{
   "\\KEEErase Raster",
   "\\KESSelect Raster",
   "\\-",
   "Select Pen Color",
   "File Access",
   "\\KEBBuilder Settings",
   "\\=",
   "\\KECEXIT"
};

static xrMenuInfo mainMenuInfo = 
{
   "Raster Builder",
   menuItems,
   8,
   NULL,
   0
};


static xrMenu * subMenu;
static INT8 * subMenuItems [2] =
{
   "\\KERRead File",
   "\\KEWWrite File",
};

static xrMenuInfo subMenuInfo = 
{
   NULL,
   subMenuItems,
   2,
   NULL,
   1
};


xrMenuIndex subMenuIndex = 
{
   NULL,
   4,
   0,
};




static xrMenu * subMenu1;
static INT8 * subMenuItems1 [8] =
{
   "Black",
   "White",
   "\\DARed",
   "\\DAGreen",
   "\\DABlue",
   "\\DACyan",
   "\\DAMagenta",
   "\\DAYellow",
};

static xrMenuInfo subMenuInfo1 = 
{
   NULL,
   subMenuItems1,
   8,
   NULL,
   2
};


xrMenuIndex subMenuIndex1 = 
{
   NULL,
   3,
   2,
};


extern INT16 panelRValue;

