/* this file originally came from imagen/src/imPcodes.h.
 * It has had more defines added but none taken out
 * command codes */
/* $Header: imPcodes.h,v 10.2 86/02/01 16:00:18 tony Rel $ */

#define	ASP0	128
#define	ASP1	129
#define	AM	130
#define AMP	131
#define	AMM	132

#define	AMMOVE 	133	/* not in impv yet */
#define	ASMOVE 	134	/* not in impv yet */
#define	ASET_ABS_H 	135	/* not in impv yet */
#define	ASET_REL_H 	136	/* not in impv yet */
#define	ASET_ABS_V 	137	/* not in impv yet */
#define	ASET_REL_V 	138	/* not in impv yet */
#define	ASRULE	192
#define	ABRULE	193

#define	AS	194	/* not implemented yet */
#define	AH	195
#define	AV	196
#define	AN	197

#define	ASGLY	198
#define	ABGLY	199
#define	ADELG	200
#define	ADELC	201
#define	ADELF	202
#define	AFONT	203

#define	APAGOR	204
#define	ASET_HV_SYS   205
#define	AROTMS	206

#define	AF	207
#define	ABSKIP	208
#define	AMARGIN	209
#define	ASETSP	210

#define	APUSH	211
#define	APOP	212

#define	APAGE	213
#define	ASET_PUSH_MASK	214	/* not in impv yet */
#define	AEND	219
#define	ACREATE_FAMILY 	221	/* not in impv yet */
#define	ACREATE_MAP	222	/* not in impv yet */
#define	ASET_PATH 	230	/* not in impv yet */
#define	ASET_TEXTURE 	231	/* not in impv yet */
#define	ASET_PEN 	232	/* not in impv yet */
#define	AFILL_PATH 	233	/* not in impv yet */
#define	ADRAW_PATH 	234	/* not in impv yet */
#define	ABIT_MAP 	235	/* not in impv yet */
#define	ASET_MAGNIFICATION 	236	/* not in impv yet */
#define	AFORCE_GLY_DELETE	240	/* not in impv yet */
#define	ADEFINE_MACRO	242	/* not in impv yet */
#define	AEXECUTE_MACRO	243	/* not in impv yet */

#define	ANOP	254
#define	AEOF	255
