.SH
Acknowledgements
.PP
Without Bob Scheifler of MIT's Laboratory for Computer Science, there
would be no X window system.
.PP
The list of contributors is now too long for an exhaustive list, and
includes Paul Asente, of Stanford University,
Mark Vandevoorde, Tony Della Fera, working for Digital at Project Athena,
Ron Newman of Project Athena,
the 
.UX
Engineering Group and the Workstations group of Digital.
My thanks also go to Sam Leffler,
Steve Miller and Noah Mendelsohn for helpful comments
during the writing of this paper.
My thanks also go to Branco Gerovac for Figure 1.
