/* $Header: vssite.h,v 10.4 86/02/01 15:48:07 tony Rel $ */
/* vssite.h
**
** This is where any machine-specific information should be stored so that
** different sites can have control over where things live.
**
*/

#define DEFAULT_FONT_DIRECTORY	"/usr/new/lib/X/font/"
#define DEFAULT_FONT_SUFFIX	".onx"

#define LOAD_FILE_2B "/usr/new/lib/X/s-code/s-code.2b"
#define LOAD_FILE_3_8 "/usr/new/lib/X/s-code/s-code.3.8"
#define LOAD_FILE_3_10 "/usr/new/lib/X/s-code/vs100fw.235"
#define LOAD_FILE_SBO "/usr/new/lib/X/s-code/s-code.sbo"
