.TL
Problems Implementing Window Systems in
.UX
.AU
James Gettys
.AI
Digital Equipment Corporation
Project Athena
Massachusetts Institute of Technology
Cambridge, Massachusetts 02139
.AB
Over that last 18 months the X window system has been implemented
under 4.2BSD
.UX
at MIT at the Laboratory for Computer Science and Project Athena.
While on the whole the result X's design is reasonably clean and pleasing,
.UX
strongly limited the possible implementation strategies.
This paper discusses the problems encountered,
how they influenced our design decisions,
and suggests issues for future study and development as 
.UX
evolves toward
a distributed environment.
.AE
