/* $Header: xsite.h,v 10.3 86/12/17 18:02:36 swick Exp $ */
/* Copyright 1985 Massachusetts Institute of Technology */

/* xsite.h
 *
 * This is where any machine or display specific information should
 * be stored so that different sites can have control over where things live.
 *
 *
 *		Copyright (c) 1986 Brown University
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies, and that both
 * that copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Brown University not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission. Brown University makes no
 * representations about the suitability of this software for any purpose.
 * It is provided "as-is" without express or implied warranty.
 */

#define DEFAULT_FONT_DIRECTORY  "/usr/new/lib/X/font/"
#define DEFAULT_FONT_SUFFIX     ".onx"
