.SH
Acknowledgements
.XS
Acknowledgements
.XE
.PP
X is the work of many people.
First and
foremost, Bob Scheifler of MIT's Laboratory for Computer Science, who
shared with me the problem of having VS100 displays without software
to make them useful.
Secondly, Paul Asente of Stanford University
and Digital's Western Research Labs (who wrote W, predecessor to X,
and is now an X contributor) and Chris Kent of Purdue University and
DECWRL who struggled to make the Vs100 useful before its time had come.
.PP
Other major contributions have been made by Tony Della Fera, Mark
Vandevoorde, Ron Newman, Yakim Martillo (MIT-Project Athena),
Doug Mink (SAO),  Shane Hartman, Stuart Malone (MIT-LCS),
Paul Johnson (MIT-LCS), and Stephen Sutphen (University of Alberta),
Eric Cooper, (CMU), and Jim Fulton (Cognition).
.PP
Both Digital's Ultrix Engineering Group and Workstation Group
must be thanked for large amounts of work on the QVSS and QDSS
impementations
and for the faith shown in an (originally) flaky research project.
.PP
Mike Braca et al. of Brown University
are responsible for the IBM RT/PC ACIS display implementation.
.PP
Dave Rosenthal, formerly of CMU's ITC, now of Sun Microsystems,
must be thanked for both many constructive comments and criticisms during
X development, and the Sun implementation.
.PP
We hope that this list of major contributors will continue to grow.
.PP
And thanks must also go to MIT,  Digital Equipment Corporation,
and IBM for providing the environment where it could happen.
.nf
			Jim Gettys
			Digital Equipment Corporation
			MIT / Project Athena
			January 28, 1986
.fi
