/*
 *	$Source: /u1/X/DECToolkit/src/RCS/Args.h,v $
 *	$Header: Args.h,v 1.1 86/12/17 09:04:21 swick Exp $
 */


/* Defines for Command Button Input Parsing */
#define T_COMMAND_TYPE 1
#define T_COMMAND_TEXT 2
#define T_COMMAND_PIXM 3
#define T_COMMAND_FONTINFO 4
#define T_COMMAND_TXCOLOR 5
#define T_COMMAND_PIXMW 6
#define T_COMMAND_PIXMH 7
#define T_COMMAND_HITYPE 8
#define T_COMMAND_BACKGRCOLOR 9
#define T_COMMAND_BRCOLOR 10
#define T_COMMAND_BRWIDTH 11
#define T_COMMAND_SELSTATE 12
#define T_COMMAND_PROC 13
#define T_COMMAND_TAG 14
#define T_COMMAND_ONHI 15
#define T_COMMAND_OFFHI 16
#define T_COMMAND_NOTIFY 17
#define T_COMMAND_REFRESH 18

typedef struct _Targ {
	int name;
	caddr_t data;
} Targ;

#define TSetArg(arg, n, d) \
	( (arg).name = (n), \
	(arg).data = (caddr_t)(d) )
