/* $Header: dvi.h,v 10.5 86/02/01 15:44:52 tony Rel $ */

#define SETCHAR0    0
#define SET1        128
#define SETRULE     132
#define PUT1        133
#define PUTRULE     137
#define NOP         138
#define BOP         139
#define EOP         140
#define PUSH        141
#define POP         142
#define RIGHT1      143
#define RIGHT2      144
#define RIGHT3      145
#define RIGHT4      146
#define W0          147
#define W1          148
#define W2          149
#define W3          150
#define W4          151
#define X0          152
#define X1          153
#define X2          154
#define X3          155
#define X4          156
#define DOWN1       157
#define DOWN2       158
#define DOWN3       159
#define DOWN4       160
#define Y0          161
#define Y1          162
#define Y2          163
#define Y3          164
#define Y4          165
#define Z0          166
#define Z1          167
#define Z2          168
#define Z3          169
#define Z4          170
#define FNTNUM0     171
#define FNT1        235
#define FNT2        236
#define FNT3        237
#define FNT4        238
#define XXX1        239
#define XXX2        240
#define XXX3        241
#define XXX4        242
#define FNTDEF1     243
#define FNTDEF2     244
#define FNTDEF3     245
#define FNTDEF4     246
#define PRE         247
#define POST        248
#define POSTPOST    249

#define TRAILER     223     /* Trailing bytes at end of file */
