/* $Header: vssite.h,v 10.4 86/11/29 13:47:11 jg Rel $ */
/* Copyright 1985, Massachusetts Institute of Technology	*/

/* vssite.h
**
** This is where any machine or display specific information should
** be stored so that different sites can have control over where things live.
**
*/
#define	DEFAULT_FONT_PATH	"/usr/new/lib/X/font:~/font"
#define DEFAULT_FONT_SUFFIX	".onx"

